//********************************************************************
//*** ResourcePool/ResourcePoolInterface.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePoolInterface.hh,v 1.12 2005/05/07 19:50:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_RESOURCEPOOLINTERFACE_HH
#define RESOURCEPOOL_RESOURCEPOOLINTERFACE_HH

#ifndef RESOURCEPOOL_COMMAND_HH
#include "Command.hh"
#endif

namespace fatalmind {
template <class Type>
class ResourcePoolInterface {
	public:
		virtual ~ResourcePoolInterface() {
		}

		/**
		 * Returns a plain resource to be used.
		 * This has to be free()ed or failed().
		 *
		 * \exception CreateException If the ResourcePool could not find 
		 *   a valid resource within the given constraints (MaxTry, Max).
		 * \exception ... Other exceptions may arise, especially if thrown
		 *   in Factory::create_resource.
		 * 
		 * \deprecated
		 * Altought it is not really deprecated, the get(), free()
		 * fail() mechanism is not the best practice. use the execute()
		 * method instead.
		 */
		virtual typename Type::plainresource_t& get() = 0;

		/**
		 * Marks a resource as free.
		 * This resource will be re-used by get() calls. The free() method
		 * calls the postcheck() method of the
		 * resource to determine if the resource is valid.
		 *
		 * \deprecated
		 * Altought it is not really deprecated, the get(), free()
		 * fail() mechanism is not the best practice. use the execute()
		 * method instead.
		 */
		virtual bool free(const typename Type::plainresource_t&) = 0;

		/**
		 * Marks a resource as bad. The ResourcePool will throw this resource
		 * away and NOT return it to the pool of available connections.
		 *
		 * \deprecated
		 * Altought it is not really deprecated, the get(), free()
		 * fail() mechanism is not the best practice. use the execute()
		 * method instead.
		 */
		virtual bool fail(const typename Type::plainresource_t&) = 0;

		/**
		 * Executes a command with a resource from this pool.
		 * Using this method is the recommended way to use ResourcePool.
		 * It handles the processing of get()/free()/fail() calls.
		 *
		 * See Command::execute for details how to implement such
		 * commands, or see PoolType::Commands for a list of available
		 * commands for a PoolType.
		 */
		virtual void execute(Command<Type>& cmd) = 0;
};
} // namespace fatalmind

#endif
