//********************************************************************
//*** ResourcePool/mysql/SQLSelect.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.hh,v 1.15 2005/03/06 14:42:30 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_MYSQL_RESOURCEPOOL_HH
#include <ResourcePool.hh>
#endif

#ifndef RESOURCEPOOL_MYSQL_SQLSELECT_HH
#define RESOURCEPOOL_MYSQL_SQLSELECT_HH

#ifndef RESOURCEPOOL_COMMAND_HH
#include "ResourcePool/Command.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_MYSQLFWD_HH
#include "mysqlFwd.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_HH
#include "SQLType.hh"
#endif

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#ifndef RESOURCEPOOL_SQL_BIND_HH
#include "ResourcePool/SQL/Bind.hh"
#endif

#ifndef RESOURCEPOOL_SQL_BINDOUT_HH
#include "ResourcePool/SQL/BindOut.hh"
#endif

#ifndef RESOURCEPOOL_SQL_SQLSELECT_HH
#include "ResourcePool/SQL/SQLSelect.hh"
#endif

#include <string>
#include <vector>

namespace fatalmind {
namespace mysql {

namespace internal {
class registerFinish;
}

/**
 * A class to select an arbitary number of records from an
 * mysql database.
 */
class SQLSelect
: public Command<fatalmind::ResourceType::mysql> 
, public SQL::Bind<SQLType, SQLTypeImpl>
, public SQL::BindOut<SQLRefType, SQLRefTypeImpl>
, public fatalmind::SQLSelect
{
	public:
		SQLSelect(const std::string& SQL);
		SQLSelect(const ResourcePool<ResourceType>& p, const std::string& SQL);
		virtual ~SQLSelect();
		virtual void execute(MYSQL&);

	protected:
		std::string _SQL;
	private:
		virtual bool fetch();
		void finish();
		void close_rs();
	
		MYSQL_RES* _rs;
		unsigned int _num_fields;
	friend
		class internal::registerFinish;
};

namespace internal {
class registerFinish {
	public:
		registerFinish(SQLSelect& select)
		: _select(select)
		{
		}
		~registerFinish()
		{
			_select.finish();
		}
	private:
		SQLSelect& _select;
};
} // namespace internal


} // namespace mysql
} // namespace fatalmind
#endif
